// $Id$

/**
 * @file
 * Override Drupal tabledrag methods to add support for nested fieldgroups.
 */

/**
 * Adjust the classes associate with a row's weight when it is dropped.
 * Use the parent's name to base the change on.
 */
Drupal.tableDrag.prototype.onDrop = function () {
  tableDragObject = this;
  var siblingRowUp = $(tableDragObject.rowObject.element).prev('tr');
  var siblingRowsDown = $(tableDragObject.rowObject.element).nextAll('tr');
  var weightField = $('input.field-weight', tableDragObject.rowObject.element);//this might not work once hidden
  var siblingWeightFieldDown = $('input.field-weight', siblingRowsDown);
  //var siblingWeightDown = siblingWeightFieldDown[0].value;
  var oldWeightField = $('input.field-weight', tableDragObject.rowObject.element);
  var previousRow = $(tableDragObject.rowObject.element).prev('tr');
  
  
  while (previousRow.length && $('.indentation', previousRow).length >= this.rowObject.indents) {
    previousRow = previousRow.prev('tr');
  }
  // If we found a row.
  if (previousRow.length) {
    sourceRow = previousRow[0];
    var oldGroupField = $('input.field-name', sourceRow);
    var newGroupName = oldGroupField[0].value.replace(/_/g, '-');
  }
  // Otherwise we went all the way to the left of the table without finding
  // a parent, meaning this item has been placed at the root level.
  else {
    var newGroupName = "top";
  }
  
  var oldGroupName = oldWeightField[0].className.replace(/([^ ]+[ ]+)*field-weight-([^ ]+)([ ]+[^ ]+)*/, '$2');
  var oldClass = 'field-weight-' + oldGroupName;
  var newClass = 'field-weight-' + newGroupName;
  weightField.removeClass(oldClass).addClass(newClass);
  
  //Now that we've updated the class names, let's make sure the weight is properly set.
  //First make sure that if the rows above are nested deeper, that we find their parent, which is a sibling of
  //our current row. 
  while (siblingRowUp.length && $('.indentation', siblingRowUp).length > this.rowObject.indents) {
    siblingRowUp = siblingRowUp.prev('tr');
  }
  //before we set the new weight, make sure we know whether the previous row is a sibling or a parent
  var siblingIndent = $('.indentation', siblingRowUp).length;
  var indents = this.rowObject.indents;
  
  //if we found a sibling
  if (siblingIndent == indents) {
    //the previous row is a sibling, so let's set the new weight
    var siblingWeightFieldUp = $('input.field-weight', siblingRowUp);
    var siblingWeightUp = siblingWeightFieldUp[0].value;
    var newWeight = parseInt(siblingWeightUp) + 1;
    $('input.field-weight', tableDragObject.rowObject.element).val(newWeight);
    
    //now deal with the case where we moved left/right
    if (oldClass != newClass) {
      $(tableDragObject.rowObject.element).nextAll('tr').each(function(){
        if ($('input.field-weight', this).hasClass(newClass)) {
          //let's make sure we grab an actual sibling
          var prevRow = $(this).prev('tr');
          //alert("Prevrow length: " + prevRow.length);
          //alert("This indents: " + $('.indentation', this).length);
          //alert("Indentation length? " + $('.indentation', prevRow).length);
          while (prevRow.length && $('.indentation', prevRow).length > $('.indentation', this).length) {
            prevRow = prevRow.prev('tr');
          }
          var prevWeight = $('input.field-weight', prevRow).val();
          var incWeight = parseInt(prevWeight) + 1;
          $('input.field-weight', this).val(incWeight);
          }
      });
    }
  }
  else {
    //the previous row is a parent, which means we're at the top of this group, so set the index to zero
    $('input.field-weight', tableDragObject.rowObject.element).val(0);
    
    //find the row's group to pass in to the each function for comparison since we only want to affect the
    //weights in this same group
    $(tableDragObject.rowObject.element).nextAll('tr').each(function(){
      if ($('input.field-weight', this).hasClass(newClass)) {
        //let's make sure we grab an actual sibling
        var prevRow = $(this).prev('tr');
        //alert("Prevrow length: " + prevRow.length);
        //alert("This indents: " + $('.indentation', this).length);
        //alert("Indentation length? " + $('.indentation', prevRow).length);
        while (prevRow.length && $('.indentation', prevRow).length > $('.indentation', this).length) {
          prevRow = prevRow.prev('tr');
        }
        var prevWeight = $('input.field-weight', prevRow).val();
        var incWeight = parseInt(prevWeight) + 1;
        $('input.field-weight', this).val(incWeight);
      }
    });
  }
  //TODO reorder weights from previous group
  
  //debug code for outputting object elements
  /*var str;
  var thingy = siblingIndent;
  for (item in thingy) {
    if (String(thingy[item]).search(/function/) == -1) {
      str += item + " = " + thingy[item] + "   ";
    }
  }
  //alert(str);
  var str2;
  var thingy2 = siblingWeightFieldDown[0].value;
  for (item2 in thingy2) {
    if (String(thingy2[item2]).search(/function/) == -1) {
      str2 += item2 + " = " + thingy2[item2] + "   ";
    }
  }*/
  //alert(siblingIndent);
  //alert("Old group: " + oldGroupName);
  //alert("New group: " + newGroupName);
}